package ho.tests.thread.synchro.test1.order3.barrier;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Semaphore;


/**
 * Thread launching example using semaphore and barrier synchronization
 * @author Huseyin OZVEREN
 */
public class ApplicationBarrierSemaphore {
	public static void main(String[] args) {
		ExecutorService es = Executors.newFixedThreadPool(2);
		// SEMPAHORE
		// Param1=current value of the semaphore that defines the number of resources available initially (when is 0 then lock)
		// Param2=boolean indicating whether the method of administration must be FIFO
		Semaphore personalSem = new Semaphore(1, true);
		Semaphore neighboringSem = new Semaphore(0, true);
		
		// BARRIER for 2 threads
		// Param1=blocking counter initialized with an integer value
		CountDownLatch barrier = new CountDownLatch(2);
		
		System.out.println( "Starting threads in main thread" );

		// create and name each runnable
		// 1st thread
		MyTask th1 = new MyTask("threadname_1", personalSem, neighboringSem, barrier);
		es.execute(th1);

		// 2nd thread
		MyTask th2 = new MyTask("threadname_2", neighboringSem, personalSem, barrier);
		es.execute(th2);
		
		try {
			barrier.await(); // Current thread is blocked until all others threads have call the method bar.countDown();
		} catch (InterruptedException ex) {
			System.out.println(ex);
		}

		// shutdown worker threads  
		es.shutdown();

		System.out.println( "Threads started, main thread ends\n" );
	}
}