package ho.tests.thread.synchro.test1.order3.barrier;

import java.util.Random;
import java.util.concurrent.Semaphore;
import java.util.concurrent.CountDownLatch;

/**
 * Mytask Runnable example using semaphore and barrier synchronization
 * @author Huseyin OZVEREN
 */
public class MyTask implements Runnable {
	private int sleepTime; // random sleep time for thread   
	private String threadName; // name of thread   
	private static Random generator = new Random();
	
	// Semaphores
	Semaphore personalSem;
	Semaphore neighboringSem;
	
	// Barrier
	CountDownLatch barrier; 
	
	public MyTask(String name, Semaphore personalSem, Semaphore neighboringSem, CountDownLatch barrier) {
		// set name of thread
		this.threadName = name;
		// random sleep time between 0 and 5 seconds
		this.sleepTime = generator.nextInt( 5000 ); 
		//
		this.personalSem = personalSem;
		this.neighboringSem = neighboringSem;
		//
		this.barrier = barrier;
	}

	public void run() {
		try {
			personalSem.acquire();
		} catch (InterruptedException ex) {
			System.out.println(ex);
		}

		try{ 
			// put thread to sleep for sleepTime amount of time
			System.out.printf( "%s starting and going to sleep for %d milliseconds.\n", this.threadName, this.sleepTime );
			// put thread to sleep
			Thread.sleep( this.sleepTime );   
		} catch ( InterruptedException exception )  {  
			exception.printStackTrace(); 
		}
		
		// print thread name
		System.out.printf( "%s done sleeping\n", this.threadName ); 
		
		// decrements the counter value of the barrier "bar"
		barrier.countDown();
		
		// release semaphore => unblock the next task
		neighboringSem.release();
	}
		
}