package ho.tests.thread.synchro.test1.order4.mutex;

import java.util.Random;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.locks.ReentrantLock;

/**
 * Mytask Runnable example using mutex and barrier synchronization
 * @author Huseyin OZVEREN
 */
public class MyTask implements Runnable {
	private int sleepTime; // random sleep time for thread   
	private String threadName; // name of thread   
	private static Random generator = new Random();

	// Mutex
	ReentrantLock mutex;
	// Barrier
	CountDownLatch barrier; 
	
	public MyTask(String name, ReentrantLock mutex, CountDownLatch barrier) {
		// set name of thread
		this.threadName = name;
		// random sleep time between 0 and 5 seconds
		this.sleepTime = generator.nextInt( 5000 ); 
		//
		this.mutex = mutex;
		//
		this.barrier = barrier;
	}

	public void run() {
		// LOCK
		mutex.lock();

		try {
			// put thread to sleep for sleepTime amount of time
			System.out.printf( "%s starting and going to sleep for %d milliseconds.\n", this.threadName, this.sleepTime );
			// put thread to sleep
			Thread.sleep( this.sleepTime );   
			
			// Instruction to protect = "critical section"
			System.out.printf("%s before modif Application.objToProtectSynchro='%s'\n", this.threadName, ApplicationBarrierMutex.objToProtectSynchro);

			// Modify the "critical resource"
			ApplicationBarrierMutex.objToProtectSynchro = "value fixed by Task "+this.threadName;
			
			System.out.printf("%s after modif Application.objToProtectSynchro='%s'\n", this.threadName, ApplicationBarrierMutex.objToProtectSynchro);
		} catch (InterruptedException exception) {
			exception.printStackTrace();
		} finally {
			// UNLOCK
			mutex.unlock();

			// decrements the counter value of the barrier "bar"
			barrier.countDown();
		}
		
		// print thread name
		System.out.printf( "%s done sleeping\n", this.threadName ); 
	}
}